@@ -77,7 +77,7 @@ const char *dtmf_trigger_types[__NUM_DTMF_TRIGGERS] = {
 bool dtmf_init(void) {
 	ilog(LOG_DEBUG, "log dtmf over ng %d", rtpe_config.dtmf_via_ng);
 	ilog(LOG_DEBUG, "no log injected dtmf %d", rtpe_config.dtmf_no_log_injects);
-	if (open_v46_socket(&dtmf_log_sock, SOCK_DGRAM)) {
+	if (!open_v46_socket(&dtmf_log_sock, SOCK_DGRAM)) {
 		ilog(LOG_ERR, "Failed to open/connect DTMF logging socket: %s", strerror(errno));
 		return false;
 	}
@@ -117,10 +117,10 @@ static void dtmf_bencode_and_notify(struct call_media *media, unsigned int event
 	data = bencode_dictionary_add_dictionary(notify, "data");
 	tags = bencode_dictionary_add_list(data, "tags");
 
-	bencode_dictionary_add_string_len(data, "callid", call->callid.s, call->callid.len);
-	bencode_dictionary_add_string_len(data, "source_tag", ml->tag.s, ml->tag.len);
+	bencode_dictionary_add_str(data, "callid", &call->callid);
+	bencode_dictionary_add_str(data, "source_tag", &ml->tag);
 	if (ml->label.s) {
-		bencode_dictionary_add_string_len(data, "source_label", ml->label.s, ml->label.len);
+		bencode_dictionary_add_str(data, "source_label", &ml->label);
 	}
 
 	tags_ht_iter iter;
@@ -131,12 +131,12 @@ static void dtmf_bencode_and_notify(struct call_media *media, unsigned int event
 
 	bencode_dictionary_add_string(data, "type", "DTMF");
 	bencode_dictionary_add_string(data, "source_ip", sockaddr_print_buf(&fsin->address));
-	bencode_dictionary_add_integer(data, "timestamp", rtpe_now.tv_sec);
+	bencode_dictionary_add_integer(data, "timestamp", rtpe_now / 1000000);
 	bencode_dictionary_add_integer(data, "event", event);
 	bencode_dictionary_add_integer(data, "duration", ((long long) duration * (1000000LL / clockrate)) / 1000LL);
 	bencode_dictionary_add_integer(data, "volume", volume);
 
-	bencode_collapse_str(notify, &encoded_data);
+	encoded_data = bencode_collapse_str(notify);
 	notify_ng_tcp_clients(&encoded_data);
 	bencode_buffer_free(&bencbuf);
 }
@@ -177,7 +177,7 @@ static GString *dtmf_json_print(struct call_media *media, unsigned int event, un
 	g_string_append_printf(buf, "],"
 			"\"type\":\"DTMF\",\"timestamp\":%lu,\"source_ip\":\"%s\","
 			"\"event\":%u,\"duration\":%u,\"volume\":%u}",
-			(unsigned long) rtpe_now.tv_sec,
+			(unsigned long) rtpe_now / 1000000,
 			sockaddr_print_buf(&fsin->address),
 			(unsigned int) event,
 			(duration * (1000000 / clockrate)) / 1000,
@@ -210,14 +210,14 @@ static void dtmf_end_event(struct call_media *media, unsigned int event, unsigne
 	// don't add to recv list when it's injected, it can cause the list TS's to be out
 	// of order breaking the dtmf-security and letting the generated PCM frames through
 	if (!injected) {
-		struct dtmf_event *ev = g_slice_alloc0(sizeof(*ev));
+		struct dtmf_event *ev = g_new0(__typeof(*ev), 1);
 		*ev = (struct dtmf_event) { .code = 0, .ts = ts, .volume = 0 };
 		t_queue_push_tail(&media->dtmf_recv, ev);
 	}
 
 	// only add to send list if injected, a delayed send, or not being blocked
 	if (injected || !media->monologue->block_dtmf || media->monologue->dtmf_delay) {
-		struct dtmf_event *ev = g_slice_alloc0(sizeof(*ev));
+		struct dtmf_event *ev = g_new0(__typeof(*ev), 1);
 		*ev = (struct dtmf_event) { .code = 0, .ts = ts + media->monologue->dtmf_delay * clockrate / 1000,
 			.volume = 0, .block_dtmf = media->monologue->block_dtmf };
 		t_queue_push_tail(&media->dtmf_send, ev);
@@ -311,7 +311,7 @@ void dtmf_trigger_set(struct call_monologue *ml, enum dtmf_trigger_type trigger_
 			dtmf_trigger_types[trigger_type],
 			(unsigned int) (state - ml->dtmf_trigger_state), STR_FMT(s));
 
-	call_str_cpy(ml->call, &state->trigger, s);
+	state->trigger = call_str_cpy(s);
 	state->matched = 0;
 	state->inactive = inactive;
 }
@@ -497,7 +497,7 @@ static void dtmf_code_event(struct call_media *media, char event, uint64_t ts, i
 	// don't add to recv list when it's injected, it can cause the list TS's to be out
 	// of order breaking the dtmf-security and letting the generated PCM frames through
 	if (!injected) {
-		ev = g_slice_alloc0(sizeof(*ev));
+		ev = g_new0(__typeof(*ev), 1);
 		*ev = (struct dtmf_event) { .code = event, .ts = ts, .volume = volume,
 			.rand_code = '0' + (ssl_random() % 10), .index = media->dtmf_count };
 		t_queue_push_tail(&media->dtmf_recv, ev);
@@ -505,7 +505,7 @@ static void dtmf_code_event(struct call_media *media, char event, uint64_t ts, i
 
 	// only add to send list if injected, a delayed send, or not being blocked
 	if (injected || !media->monologue->block_dtmf || media->monologue->dtmf_delay) {
-		ev = g_slice_alloc0(sizeof(*ev));
+		ev = g_new0(__typeof(*ev), 1);
 		*ev = (struct dtmf_event) { .code = event, .ts = ts + media->monologue->dtmf_delay * clockrate / 1000,
 			.volume = volume,
 			.block_dtmf = media->monologue->block_dtmf };
@@ -632,7 +632,7 @@ void dtmf_dsp_event(const struct dtmf_event *new_event, struct dtmf_event *cur_e
 }
 
 void dtmf_event_free(struct dtmf_event *e) {
-	g_slice_free1(sizeof(*e), e);
+	g_free(e);
 }
 
 // returns: 0 = no DTMF. 1 = DTMF start event. 2 = DTMF in progress. 3 = DTMF end event.
@@ -715,7 +715,7 @@ int dtmf_code_from_char(char c) {
 // takes over the csh reference
 static const char *dtmf_inject_pcm(struct call_media *media, struct call_media *sink,
 		struct call_monologue *monologue,
-		struct packet_stream *ps, struct ssrc_ctx *ssrc_in, struct codec_handler *ch,
+		struct packet_stream *ps, struct ssrc_entry_call *ssrc_in, struct codec_handler *ch,
 		struct codec_ssrc_handler *csh,
 		int code, int volume, int duration, int pause)
 {
@@ -724,15 +724,14 @@ static const char *dtmf_inject_pcm(struct call_media *media, struct call_media *
 	for (__auto_type l = ps->rtp_sinks.head; l; l = l->next) {
 		struct sink_handler *sh = l->data;
 		struct packet_stream *sink_ps = sh->sink;
-		struct call_monologue *sink_ml = sink_ps->media->monologue;
-		packet_sequencer_t *seq = g_hash_table_lookup(ssrc_in->parent->sequencers, sink_ps->media);
+		__auto_type sink_media = sink_ps->media;
+		packet_sequencer_t *seq = g_hash_table_lookup(ssrc_in->sequencers, sink_ps->media);
 		if (!seq)
 			continue;
 
-		struct ssrc_ctx *ssrc_out = get_ssrc_ctx(sh->attrs.transcoding ?
-					ssrc_in->ssrc_map_out : ssrc_in->parent->h.ssrc,
-				sink_ml->ssrc_hash, SSRC_DIR_OUTPUT,
-				monologue);
+		struct ssrc_entry_call *ssrc_out = get_ssrc(sh->attrs.transcoding ?
+					ssrc_in->ssrc_map_out : ssrc_in->h.ssrc,
+				&sink_media->ssrc_hash_out);
 		if (!ssrc_out)
 			return "No output SSRC context present"; // XXX generate stream
 
@@ -751,7 +750,7 @@ static const char *dtmf_inject_pcm(struct call_media *media, struct call_media *
 			.m_pt = 0xff,
 			.timestamp = 0,
 			.seq_num = htons(seq->seq),
-			.ssrc = htonl(ssrc_in->parent->h.ssrc),
+			.ssrc = htonl(ssrc_in->h.ssrc),
 		};
 		struct media_packet packet = {
 			.tv = rtpe_now,
@@ -789,7 +788,7 @@ static const char *dtmf_inject_pcm(struct call_media *media, struct call_media *
 		media_socket_dequeue(&packet, sink_ps);
 
 		obj_put_o((struct obj *) csh);
-		ssrc_ctx_put(&ssrc_out);
+		ssrc_entry_release(ssrc_out);
 	}
 
 	return 0;
@@ -803,7 +802,7 @@ const char *dtmf_inject(struct call_media *media, int code, int volume, int dura
 	if (!media->streams.head)
 		return "Media doesn't have an RTP stream";
 	struct packet_stream *ps = media->streams.head->data;
-	struct ssrc_ctx *ssrc_in = ps->ssrc_in[0];
+	struct ssrc_entry_call *ssrc_in = call_get_first_ssrc(&media->ssrc_hash_in);
 	if (!ssrc_in)
 		return "No SSRC context present for DTMF injection"; // XXX fall back to generating stream
 
@@ -827,7 +826,7 @@ const char *dtmf_inject(struct call_media *media, int code, int volume, int dura
 		// skip DTMF PTs
 		if (pt == ch_pt)
 			continue;
-		if (ch->output_handler && ch->output_handler->ssrc_hash) // context switch if we have multiple inputs going to one output
+		if (ch->output_handler) // context switch if we have multiple inputs going to one output
 			ch = ch->output_handler;
 
 		ilog(LOG_DEBUG, "DTMF injection: Using PT %i/%i -> %i (%i), SSRC %" PRIx32,
@@ -835,11 +834,9 @@ const char *dtmf_inject(struct call_media *media, int code, int volume, int dura
 				ch->source_pt.payload_type,
 				ch->dest_pt.payload_type,
 				ch_pt,
-				ssrc_in->parent->h.ssrc);
+				ssrc_in->h.ssrc);
 
-		if (!ch->ssrc_hash)
-			continue;
-		csh = get_ssrc(ssrc_in->parent->h.ssrc, ch->ssrc_hash);
+		csh = get_ssrc(ssrc_in->h.ssrc, &ch->ssrc_hash);
 		if (!csh)
 			continue;
 		break;
@@ -849,8 +846,6 @@ const char *dtmf_inject(struct call_media *media, int code, int volume, int dura
 		return "No RTP payload type found to be in use"; // XXX generate stream
 	if (!ch)
 		return "No matching codec handler";
-	if (!ch->ssrc_hash)
-		return "No suitable codec handler present";
 	if (!csh)
 		return "No matching codec SSRC handler";
 
@@ -862,7 +857,7 @@ const char *dtmf_inject(struct call_media *media, int code, int volume, int dura
 	ilog(LOG_DEBUG, "Injecting RFC DTMF event #%i for %i ms (vol %i) from '" STR_FORMAT "' (media #%u) "
 			"into RTP PT %i, SSRC %" PRIx32,
 			code, duration, volume, STR_FMT(&monologue->tag), media->index, pt,
-			ssrc_in->parent->h.ssrc);
+			ssrc_in->h.ssrc);
 
 	// synthesise start and stop events
 	// the num_samples needs to be based on the the previous packet timestamp so we need to
