@@ -18,15 +18,15 @@
 #include "statistics.h"
 #include "main.h"
 
-struct timeval rtpe_latest_graphite_interval_start;
+int64_t rtpe_latest_graphite_interval_start;
 
 static socket_t graphite_sock;
 static int connection_state = STATE_DISCONNECTED;
 //struct totalstats totalstats_prev;
-static time_t next_run;
+static int64_t next_run;
 // HEAD: static time_t rtpe_now, next_run;
 static char* graphite_prefix = NULL;
-static struct timeval graphite_interval_tv;
+static int64_t graphite_interval_tv;
 
 struct global_stats_counter rtpe_stats_graphite_diff;		// per-interval increases
 static struct global_stats_counter rtpe_stats_graphite_intv;	// copied out when graphite stats run
@@ -44,8 +44,8 @@ static struct global_stats_sampled rtpe_sampled_graphite_min_max_intv;
 struct global_sampled_avg rtpe_sampled_graphite_avg;
 
 
-void set_graphite_interval_tv(struct timeval *tv) {
-	graphite_interval_tv = *tv;
+void set_graphite_interval_tv(int64_t tv) {
+	graphite_interval_tv = tv;
 }
 
 void set_prefix(char* prefix) {
@@ -69,7 +69,7 @@ static int connect_to_graphite_server(const endpoint_t *graphite_ep) {
 	rc = connect_socket_nb(&graphite_sock, SOCK_STREAM, graphite_ep);
 
 	if (rtpe_config.graphite_timeout > 0 && !(graphite_sock.fd < 0)) {
-		usertimeout(graphite_sock.fd, rtpe_config.graphite_timeout * 1000);
+		usertimeout(&graphite_sock, rtpe_config.graphite_timeout * 1000);
 	}
 
 	if (rc == -1) {
@@ -89,7 +89,7 @@ static int connect_to_graphite_server(const endpoint_t *graphite_ep) {
 
 GString *print_graphite_data(void) {
 
-	long long time_diff_us = timeval_diff(&rtpe_now, &rtpe_latest_graphite_interval_start);
+	int64_t time_diff_us = rtpe_now - rtpe_latest_graphite_interval_start;
 	rtpe_latest_graphite_interval_start = rtpe_now;
 
 	stats_counters_calc_diff(rtpe_stats, &rtpe_stats_graphite_intv, &rtpe_stats_graphite_diff);
@@ -109,9 +109,9 @@ GString *print_graphite_data(void) {
 #define GPF(fmt, ...) \
 	if (graphite_prefix) \
 		g_string_append(graph_str, graphite_prefix); \
-	g_string_append_printf(graph_str, fmt " %llu\n", ##__VA_ARGS__, (unsigned long long)rtpe_now.tv_sec)
+	g_string_append_printf(graph_str, fmt " %" PRId64 "\n", ##__VA_ARGS__, rtpe_now / 1000000L)
 
-	for (int i = 0; i < NGC_COUNT; i++) {
+	for (int i = 0; i < OP_COUNT; i++) {
 		GPF("%s_time_min %.6f", ng_command_strings_esc[i],
 				(double) atomic64_get_na(&rtpe_sampled_graphite_min_max_sampled.min.ng_command_times[i]) / 1000000.0);
 		GPF("%s_time_max %.6f", ng_command_strings_esc[i],
@@ -119,9 +119,9 @@ GString *print_graphite_data(void) {
 		GPF("%s_time_avg %.6f", ng_command_strings_esc[i],
 				(double) atomic64_get_na(&rtpe_sampled_graphite_avg.avg.ng_command_times[i]) / 1000000.0);
 
-		GPF("%ss_ps_min " UINT64F, ng_command_strings_esc[i], atomic64_get_na(&rtpe_rate_graphite_min_max_avg_sampled.min.ng_commands[i]));
-		GPF("%ss_ps_max " UINT64F, ng_command_strings_esc[i], atomic64_get_na(&rtpe_rate_graphite_min_max_avg_sampled.max.ng_commands[i]));
-		GPF("%ss_ps_avg " UINT64F, ng_command_strings_esc[i], atomic64_get_na(&rtpe_rate_graphite_min_max_avg_sampled.avg.ng_commands[i]));
+		GPF("%ss_ps_min %" PRIu64, ng_command_strings_esc[i], atomic64_get_na(&rtpe_rate_graphite_min_max_avg_sampled.min.ng_commands[i]));
+		GPF("%ss_ps_max %" PRIu64, ng_command_strings_esc[i], atomic64_get_na(&rtpe_rate_graphite_min_max_avg_sampled.max.ng_commands[i]));
+		GPF("%ss_ps_avg %" PRIu64, ng_command_strings_esc[i], atomic64_get_na(&rtpe_rate_graphite_min_max_avg_sampled.avg.ng_commands[i]));
 
 		ilog(LOG_DEBUG, "Min/Max/Avg %s processing delay: %.6f/%.6f/%.6f sec",
 			ng_command_strings[i],
@@ -133,46 +133,46 @@ GString *print_graphite_data(void) {
 	}
 
 	GPF("call_dur %.6f", (double) atomic64_get_na(&rtpe_stats_graphite_diff.total_calls_duration_intv) / 1000000.0);
-	struct timeval avg_duration;
+	int64_t avg_duration;
 	uint64_t managed_sess = atomic64_get_na(&rtpe_stats_graphite_diff.managed_sess);
 	if (managed_sess)
-		timeval_from_us(&avg_duration, atomic64_get_na(&rtpe_stats_graphite_diff.call_duration) / managed_sess);
+		avg_duration = atomic64_get_na(&rtpe_stats_graphite_diff.call_duration) / managed_sess;
 	else
-		avg_duration = (struct timeval) {0,0};
-	GPF("average_call_dur %llu.%06llu",(unsigned long long)avg_duration.tv_sec,(unsigned long long)avg_duration.tv_usec);
-	GPF("forced_term_sess "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.forced_term_sess));
-	GPF("managed_sess "UINT64F, atomic64_get_na(&rtpe_stats->managed_sess));
-	GPF("managed_sess_min "UINT64F, atomic64_get_na(&rtpe_gauge_graphite_min_max_sampled.min.total_sessions));
-	GPF("managed_sess_max "UINT64F, atomic64_get_na(&rtpe_gauge_graphite_min_max_sampled.max.total_sessions));
-	GPF("current_sessions_total "UINT64F, atomic64_get_na(&rtpe_stats_gauge.total_sessions));
-	GPF("current_sessions_own "UINT64F, atomic64_get_na(&rtpe_stats_gauge.total_sessions) - atomic64_get_na(&rtpe_stats_gauge.foreign_sessions));
-	GPF("current_sessions_foreign "UINT64F, atomic64_get_na(&rtpe_stats_gauge.foreign_sessions));
-	GPF("current_transcoded_media "UINT64F, atomic64_get_na(&rtpe_stats_gauge.transcoded_media));
-	GPF("current_sessions_ipv4 "UINT64F, atomic64_get_na(&rtpe_stats_gauge.ipv4_sessions));
-	GPF("current_sessions_ipv6 "UINT64F, atomic64_get_na(&rtpe_stats_gauge.ipv6_sessions));
-	GPF("current_sessions_mixed "UINT64F, atomic64_get_na(&rtpe_stats_gauge.mixed_sessions));
-	GPF("nopacket_relayed_sess "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.nopacket_relayed_sess));
-	GPF("oneway_stream_sess "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.oneway_stream_sess));
-	GPF("regular_term_sess "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.regular_term_sess));
-	GPF("relayed_errors_user "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.errors_user));
-	GPF("relayed_packets_user "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.packets_user));
-	GPF("relayed_bytes_user "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.bytes_user));
-	GPF("relayed_errors_kernel "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.errors_kernel));
-	GPF("relayed_packets_kernel "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.packets_kernel));
-	GPF("relayed_bytes_kernel "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.bytes_kernel));
-	GPF("relayed_errors "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.errors_user) +
+		avg_duration = 0;
+	GPF("average_call_dur %" PRId64 ".%06" PRId64, avg_duration / 1000000, avg_duration % 1000000);
+	GPF("forced_term_sess %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.forced_term_sess));
+	GPF("managed_sess %" PRIu64, atomic64_get_na(&rtpe_stats->managed_sess));
+	GPF("managed_sess_min %" PRIu64, atomic64_get_na(&rtpe_gauge_graphite_min_max_sampled.min.total_sessions));
+	GPF("managed_sess_max %" PRIu64, atomic64_get_na(&rtpe_gauge_graphite_min_max_sampled.max.total_sessions));
+	GPF("current_sessions_total %" PRIu64, atomic64_get_na(&rtpe_stats_gauge.total_sessions));
+	GPF("current_sessions_own %" PRIu64, atomic64_get_na(&rtpe_stats_gauge.total_sessions) - atomic64_get_na(&rtpe_stats_gauge.foreign_sessions));
+	GPF("current_sessions_foreign %" PRIu64, atomic64_get_na(&rtpe_stats_gauge.foreign_sessions));
+	GPF("current_transcoded_media %" PRIu64, atomic64_get_na(&rtpe_stats_gauge.transcoded_media));
+	GPF("current_sessions_ipv4 %" PRIu64, atomic64_get_na(&rtpe_stats_gauge.ipv4_sessions));
+	GPF("current_sessions_ipv6 %" PRIu64, atomic64_get_na(&rtpe_stats_gauge.ipv6_sessions));
+	GPF("current_sessions_mixed %" PRIu64, atomic64_get_na(&rtpe_stats_gauge.mixed_sessions));
+	GPF("nopacket_relayed_sess %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.nopacket_relayed_sess));
+	GPF("oneway_stream_sess %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.oneway_stream_sess));
+	GPF("regular_term_sess %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.regular_term_sess));
+	GPF("relayed_errors_user %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.errors_user));
+	GPF("relayed_packets_user %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.packets_user));
+	GPF("relayed_bytes_user %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.bytes_user));
+	GPF("relayed_errors_kernel %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.errors_kernel));
+	GPF("relayed_packets_kernel %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.packets_kernel));
+	GPF("relayed_bytes_kernel %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.bytes_kernel));
+	GPF("relayed_errors %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.errors_user) +
 			atomic64_get_na(&rtpe_stats_graphite_diff.errors_kernel));
-	GPF("relayed_packets "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.packets_user) +
+	GPF("relayed_packets %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.packets_user) +
 			atomic64_get_na(&rtpe_stats_graphite_diff.packets_kernel));
-	GPF("relayed_bytes "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.bytes_user) +
+	GPF("relayed_bytes %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.bytes_user) +
 			atomic64_get_na(&rtpe_stats_graphite_diff.bytes_kernel));
-	GPF("silent_timeout_sess "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.silent_timeout_sess));
-	GPF("final_timeout_sess "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.final_timeout_sess));
-	GPF("offer_timeout_sess "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.offer_timeout_sess));
-	GPF("timeout_sess "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.timeout_sess));
-	GPF("reject_sess "UINT64F, atomic64_get_na(&rtpe_stats_graphite_diff.rejected_sess));
+	GPF("silent_timeout_sess %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.silent_timeout_sess));
+	GPF("final_timeout_sess %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.final_timeout_sess));
+	GPF("offer_timeout_sess %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.offer_timeout_sess));
+	GPF("timeout_sess %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.timeout_sess));
+	GPF("reject_sess %" PRIu64, atomic64_get_na(&rtpe_stats_graphite_diff.rejected_sess));
 
-	for (GList *l = all_local_interfaces.head; l; l = l->next) {
+	for (__auto_type l = all_local_interfaces.head; l; l = l->next) {
 		struct local_intf *lif = l->data;
 		// only show first-order interface entries: socket families must match
 		if (lif->logical->preferred_family != lif->spec->local_address.addr.family)
@@ -180,15 +180,15 @@ GString *print_graphite_data(void) {
 		int num_ports = lif->spec->port_pool.max - lif->spec->port_pool.min + 1;
 		GPF("ports_free_%s_%s %i", lif->logical->name.s,
 				sockaddr_print_buf(&lif->spec->local_address.addr),
-				g_hash_table_size(lif->spec->port_pool.free_ports_ht));
+				lif->spec->port_pool.free_ports_q.length);
 		GPF("ports_used_%s_%s %i", lif->logical->name.s,
 				sockaddr_print_buf(&lif->spec->local_address.addr),
-				num_ports - g_hash_table_size(lif->spec->port_pool.free_ports_ht));
+				num_ports - lif->spec->port_pool.free_ports_q.length);
 	}
 
 	mutex_lock(&rtpe_codec_stats_lock);
 
-	int last_tv_sec = rtpe_now.tv_sec - 1;
+	int last_tv_sec = rtpe_now / 1000000 - 1;
 	unsigned int idx = last_tv_sec & 1;
 
 	codec_stats_ht_iter iter;
@@ -211,11 +211,11 @@ GString *print_graphite_data(void) {
 	mutex_unlock(&rtpe_codec_stats_lock);
 
 
-	ilog(LOG_DEBUG, "min_sessions:%llu max_sessions:%llu, call_dur_per_interval:%.6f at time %llu\n",
+	ilog(LOG_DEBUG, "min_sessions:%llu max_sessions:%llu, call_dur_per_interval:%.6f at time %" PRId64 "\n",
 			(unsigned long long) atomic64_get_na(&rtpe_gauge_graphite_min_max_sampled.min.total_sessions),
 			(unsigned long long) atomic64_get_na(&rtpe_gauge_graphite_min_max_sampled.max.total_sessions),
 			(double) atomic64_get_na(&rtpe_stats_graphite_diff.total_calls_duration_intv) / 1000000.0,
-			(unsigned long long ) rtpe_now.tv_sec);
+			rtpe_now / 1000000);
 
 	return graph_str;
 }
@@ -232,7 +232,7 @@ static int send_graphite_data(void) {
 	size_t sent = 0;
 	int blockings = 10; // let it block that many times
 	while (sent < graph_str->len) {
-		int rc = write(graphite_sock.fd, graph_str->str + sent, graph_str->len - sent);
+		ssize_t rc = write(graphite_sock.fd, graph_str->str + sent, graph_str->len - sent);
 		if (rc<0) {
 			if (blockings <= 0 || (errno != EWOULDBLOCK && errno != EAGAIN && errno != EINTR)) {
 				ilog(LOG_ERROR,"Could not write to graphite socket (%s). " \
@@ -260,7 +260,7 @@ error:
 }
 
 
-static void graphite_loop_run(endpoint_t *graphite_ep, int seconds) {
+static void graphite_loop_run(endpoint_t *graphite_ep, int64_t microseconds) {
 
 	int rc=0;
 	struct pollfd wfds[1];
@@ -304,22 +304,22 @@ static void graphite_loop_run(endpoint_t *graphite_ep, int seconds) {
 		}
 	}
 
-	gettimeofday(&rtpe_now, NULL);
-	if (rtpe_now.tv_sec < next_run) {
-		usleep(100000);
+	rtpe_now = now_us();
+	if (rtpe_now < next_run) {
+		usleep(100000); // XXX refactor using proper sleep time
 		return;
 	}
 
-	next_run = rtpe_now.tv_sec + seconds;
+	next_run = rtpe_now + microseconds;
 
 	if (graphite_sock.fd < 0 && connection_state == STATE_DISCONNECTED) {
 		connect_to_graphite_server(graphite_ep);
 	}
 
 	if (graphite_sock.fd >= 0 && connection_state == STATE_CONNECTED) {
-		add_total_calls_duration_in_interval(&graphite_interval_tv);
+		add_total_calls_duration_in_interval(graphite_interval_tv);
 
-		gettimeofday(&rtpe_now, NULL);
+		rtpe_now = now_us();
 		rc = send_graphite_data();
 		if (rc < 0) {
 			ilog(LOG_ERROR,"Sending graphite data failed.");
@@ -339,5 +339,5 @@ void graphite_loop(void *d) {
 	connect_to_graphite_server(&rtpe_config.graphite_ep);
 
 	while (!rtpe_shutdown)
-		graphite_loop_run(&rtpe_config.graphite_ep, rtpe_config.graphite_interval); // time in seconds
+		graphite_loop_run(&rtpe_config.graphite_ep, rtpe_config.graphite_interval * 1000000LL);
 }
