@@ -9,7 +9,7 @@
 #include "control_ng.h"
 #include "bufferpool.h"
 
-struct timeval rtpe_started;
+int64_t rtpe_started;
 
 
 mutex_t rtpe_codec_stats_lock;
@@ -182,9 +182,9 @@ found:;
 
 			RTPE_STATS_INC(managed_sess);
 
-			if (!c->destroyed.tv_sec)
+			if (!c->destroyed)
 				c->destroyed = rtpe_now;
-			long long duration = timeval_diff(&c->destroyed, &c->created);
+			int64_t duration = c->destroyed - c->created;
 			RTPE_STATS_ADD(call_duration, duration);
 			duration /= 1000; // millisecond precision for the squared value to avoid overflows
 			RTPE_STATS_ADD(call_duration2, duration * duration);
@@ -233,7 +233,7 @@ INLINE void metric_push(stats_metric_q *ret, stats_metric *m) {
 static void add_metric(stats_metric_q *ret, const char *label, const char *desc, const char *fmt1, const char *fmt2, ...) {
 	va_list ap;
 
-	stats_metric *m = g_slice_alloc0(sizeof(*m));
+	stats_metric *m = g_new0(__typeof(*m), 1);
 	if (label)
 		m->label = g_strdup(label);
 	if (desc)
@@ -276,7 +276,7 @@ static void add_metric(stats_metric_q *ret, const char *label, const char *desc,
 static void add_header(stats_metric_q *ret, const char *fmt1, const char *fmt2, ...) {
 	va_list ap;
 
-	stats_metric *m = g_slice_alloc0(sizeof(*m));
+	stats_metric *m = g_new0(__typeof(*m), 1);
 	if (fmt1) {
 		va_start(ap, fmt2); // coverity[copy_paste_error : FALSE]
 		m->label = g_strdup_vprintf(fmt1, ap);
@@ -327,50 +327,54 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 	cur_sessions = t_hash_table_size(rtpe_callhash);
 	rwlock_unlock_r(&rtpe_callhash_lock);
 
-	METRIC("sessionsown", "Owned sessions", UINT64F, UINT64F, cur_sessions - atomic64_get_na(&rtpe_stats_gauge.foreign_sessions));
+	METRIC("sessionsown", "Owned sessions", "%" PRIu64, "%" PRIu64, cur_sessions - atomic64_get_na(&rtpe_stats_gauge.foreign_sessions));
 	PROM("sessions", "gauge");
 	PROMLAB("type=\"own\"");
-	METRIC("sessionsforeign", "Foreign sessions", UINT64F, UINT64F, atomic64_get_na(&rtpe_stats_gauge.foreign_sessions));
+	METRIC("sessionsforeign", "Foreign sessions", "%" PRIu64, "%" PRIu64, atomic64_get_na(&rtpe_stats_gauge.foreign_sessions));
 	PROM("sessions", "gauge");
 	PROMLAB("type=\"foreign\"");
 
-	METRIC("sessionstotal", "Total sessions", UINT64F, UINT64F, cur_sessions);
-	METRIC("transcodedmedia", "Transcoded media", UINT64F, UINT64F, atomic64_get_na(&rtpe_stats_gauge.transcoded_media));
+	METRIC("sessionstotal", "Total sessions", "%" PRIu64, "%" PRIu64, cur_sessions);
+	METRIC("transcodedmedia", "Transcoded media", "%" PRIu64, "%" PRIu64, atomic64_get_na(&rtpe_stats_gauge.transcoded_media));
 	PROM("transcoded_media", "gauge");
+	METRIC("mediacache", "Media cache size", "%" PRIu64, "%" PRIu64, atomic64_get_na(&rtpe_stats_gauge.media_cache));
+	PROM("media_cache", "gauge");
+	METRIC("playercache", "Player cache size", "%" PRIu64, "%" PRIu64, atomic64_get_na(&rtpe_stats_gauge.player_cache));
+	PROM("player_cache", "gauge");
 
-	METRIC("packetrate_user", "Packets per second (userspace)", UINT64F, UINT64F,
+	METRIC("packetrate_user", "Packets per second (userspace)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats_rate.packets_user));
-	METRIC("byterate_user", "Bytes per second (userspace)", UINT64F, UINT64F,
+	METRIC("byterate_user", "Bytes per second (userspace)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats_rate.bytes_user));
-	METRIC("errorrate_user", "Errors per second (userspace)", UINT64F, UINT64F,
+	METRIC("errorrate_user", "Errors per second (userspace)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats_rate.errors_user));
-	METRIC("packetrate_kernel", "Packets per second (kernel)", UINT64F, UINT64F,
+	METRIC("packetrate_kernel", "Packets per second (kernel)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats_rate.packets_kernel));
-	METRIC("byterate_kernel", "Bytes per second (kernel)", UINT64F, UINT64F,
+	METRIC("byterate_kernel", "Bytes per second (kernel)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats_rate.bytes_kernel));
-	METRIC("errorrate_kernel", "Errors per second (kernel)", UINT64F, UINT64F,
+	METRIC("errorrate_kernel", "Errors per second (kernel)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats_rate.errors_kernel));
-	METRIC("packetrate", "Packets per second (total)", UINT64F, UINT64F,
+	METRIC("packetrate", "Packets per second (total)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats_rate.packets_user) +
 			atomic64_get_na(&rtpe_stats_rate.packets_kernel));
-	METRIC("byterate", "Bytes per second (total)", UINT64F, UINT64F,
+	METRIC("byterate", "Bytes per second (total)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats_rate.bytes_user) +
 			atomic64_get_na(&rtpe_stats_rate.bytes_kernel));
-	METRIC("errorrate", "Errors per second (total)", UINT64F, UINT64F,
+	METRIC("errorrate", "Errors per second (total)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats_rate.errors_user) +
 			atomic64_get_na(&rtpe_stats_rate.errors_kernel));
 
-	METRIC("media_userspace", "Userspace-only media streams", UINT64F, UINT64F,
+	METRIC("media_userspace", "Userspace-only media streams", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats_gauge.userspace_streams));
 	PROM("mediastreams", "gauge");
 	PROMLAB("type=\"userspace\"");
 
-	METRIC("media_kernel", "Kernel-only media streams", UINT64F, UINT64F,
+	METRIC("media_kernel", "Kernel-only media streams", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats_gauge.kernel_only_streams));
 	PROM("mediastreams", "gauge");
 	PROMLAB("type=\"kernel\"");
 
-	METRIC("media_mixed", "Mixed kernel/userspace media streams", UINT64F, UINT64F,
+	METRIC("media_mixed", "Mixed kernel/userspace media streams", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats_gauge.kernel_user_streams));
 	PROM("mediastreams", "gauge");
 	PROMLAB("type=\"mixed\"");
@@ -383,72 +387,72 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 	HEADER("totalstatistics", "Total statistics (does not include current running sessions):");
 	HEADER("{", "");
 
-	METRIC("uptime", "Uptime of rtpengine", "%llu", "%llu seconds", (long long) timeval_diff(&rtpe_now, &rtpe_started) / 1000000);
+	METRIC("uptime", "Uptime of rtpengine", "%" PRId64, "%" PRId64 " seconds", (rtpe_now - rtpe_started) / 1000000);
 	PROM("uptime_seconds", "gauge");
 
-	METRIC("managedsessions", "Total managed sessions", UINT64F, UINT64F, num_sessions);
+	METRIC("managedsessions", "Total managed sessions", "%" PRIu64, "%" PRIu64, num_sessions);
 	PROM("sessions_total", "counter");
-	METRIC("rejectedsessions", "Total rejected sessions", UINT64F, UINT64F, atomic64_get_na(&rtpe_stats->rejected_sess));
+	METRIC("rejectedsessions", "Total rejected sessions", "%" PRIu64, "%" PRIu64, atomic64_get_na(&rtpe_stats->rejected_sess));
 	PROM("closed_sessions_total", "counter");
 	PROMLAB("reason=\"rejected\"");
-	METRIC("timeoutsessions", "Total timed-out sessions via TIMEOUT", UINT64F, UINT64F, atomic64_get_na(&rtpe_stats->timeout_sess));
+	METRIC("timeoutsessions", "Total timed-out sessions via TIMEOUT", "%" PRIu64, "%" PRIu64, atomic64_get_na(&rtpe_stats->timeout_sess));
 	PROM("closed_sessions_total", "counter");
 	PROMLAB("reason=\"timeout\"");
-	METRIC("silenttimeoutsessions", "Total timed-out sessions via SILENT_TIMEOUT", UINT64F, UINT64F,atomic64_get_na(&rtpe_stats->silent_timeout_sess));
+	METRIC("silenttimeoutsessions", "Total timed-out sessions via SILENT_TIMEOUT", "%" PRIu64, "%" PRIu64,atomic64_get_na(&rtpe_stats->silent_timeout_sess));
 	PROM("closed_sessions_total", "counter");
 	PROMLAB("reason=\"silent_timeout\"");
-	METRIC("finaltimeoutsessions", "Total timed-out sessions via FINAL_TIMEOUT", UINT64F, UINT64F,atomic64_get_na(&rtpe_stats->final_timeout_sess));
+	METRIC("finaltimeoutsessions", "Total timed-out sessions via FINAL_TIMEOUT", "%" PRIu64, "%" PRIu64,atomic64_get_na(&rtpe_stats->final_timeout_sess));
 	PROM("closed_sessions_total", "counter");
 	PROMLAB("reason=\"final_timeout\"");
-	METRIC("offertimeoutsessions", "Total timed-out sessions via OFFER_TIMEOUT", UINT64F, UINT64F,atomic64_get_na(&rtpe_stats->offer_timeout_sess));
+	METRIC("offertimeoutsessions", "Total timed-out sessions via OFFER_TIMEOUT", "%" PRIu64, "%" PRIu64,atomic64_get_na(&rtpe_stats->offer_timeout_sess));
 	PROM("closed_sessions_total", "counter");
 	PROMLAB("reason=\"offer_timeout\"");
-	METRIC("regularterminatedsessions", "Total regular terminated sessions", UINT64F, UINT64F, atomic64_get_na(&rtpe_stats->regular_term_sess));
+	METRIC("regularterminatedsessions", "Total regular terminated sessions", "%" PRIu64, "%" PRIu64, atomic64_get_na(&rtpe_stats->regular_term_sess));
 	PROM("closed_sessions_total", "counter");
 	PROMLAB("reason=\"terminated\"");
-	METRIC("forcedterminatedsessions", "Total forced terminated sessions", UINT64F, UINT64F, atomic64_get_na(&rtpe_stats->forced_term_sess));
+	METRIC("forcedterminatedsessions", "Total forced terminated sessions", "%" PRIu64, "%" PRIu64, atomic64_get_na(&rtpe_stats->forced_term_sess));
 	PROM("closed_sessions_total", "counter");
 	PROMLAB("reason=\"force_terminated\"");
 
-	METRIC("relayedpackets_user", "Total relayed packets (userspace)", UINT64F, UINT64F,
+	METRIC("relayedpackets_user", "Total relayed packets (userspace)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats->packets_user));
 	PROM("packets_total", "counter");
 	PROMLAB("type=\"userspace\"");
-	METRIC("relayedpacketerrors_user", "Total relayed packet errors (userspace)", UINT64F, UINT64F,
+	METRIC("relayedpacketerrors_user", "Total relayed packet errors (userspace)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats->errors_user));
 	PROM("packet_errors_total", "counter");
 	PROMLAB("type=\"userspace\"");
-	METRIC("relayedbytes_user", "Total relayed bytes (userspace)", UINT64F, UINT64F,
+	METRIC("relayedbytes_user", "Total relayed bytes (userspace)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats->bytes_user));
 	PROM("bytes_total", "counter");
 	PROMLAB("type=\"userspace\"");
 
-	METRIC("relayedpackets_kernel", "Total relayed packets (kernel)", UINT64F, UINT64F,
+	METRIC("relayedpackets_kernel", "Total relayed packets (kernel)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats->packets_kernel));
 	PROM("packets_total", "counter");
 	PROMLAB("type=\"kernel\"");
-	METRIC("relayedpacketerrors_kernel", "Total relayed packet errors (kernel)", UINT64F, UINT64F,
+	METRIC("relayedpacketerrors_kernel", "Total relayed packet errors (kernel)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats->errors_kernel));
 	PROM("packet_errors_total", "counter");
 	PROMLAB("type=\"kernel\"");
-	METRIC("relayedbytes_kernel", "Total relayed bytes (kernel)", UINT64F, UINT64F,
+	METRIC("relayedbytes_kernel", "Total relayed bytes (kernel)", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats->bytes_kernel));
 	PROM("bytes_total", "counter");
 	PROMLAB("type=\"kernel\"");
 
-	METRIC("relayedpackets", "Total relayed packets", UINT64F, UINT64F,
+	METRIC("relayedpackets", "Total relayed packets", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats->packets_kernel) +
 			atomic64_get_na(&rtpe_stats->packets_user));
-	METRIC("relayedpacketerrors", "Total relayed packet errors", UINT64F, UINT64F,
+	METRIC("relayedpacketerrors", "Total relayed packet errors", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats->errors_kernel) +
 			atomic64_get_na(&rtpe_stats->errors_user));
-	METRIC("relayedbytes", "Total relayed bytes", UINT64F, UINT64F,
+	METRIC("relayedbytes", "Total relayed bytes", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats->bytes_kernel) +
 			atomic64_get_na(&rtpe_stats->bytes_user));
 
-	METRIC("zerowaystreams", "Total number of streams with no relayed packets", UINT64F, UINT64F, atomic64_get_na(&rtpe_stats->nopacket_relayed_sess));
+	METRIC("zerowaystreams", "Total number of streams with no relayed packets", "%" PRIu64, "%" PRIu64, atomic64_get_na(&rtpe_stats->nopacket_relayed_sess));
 	PROM("zero_packet_streams_total", "counter");
-	METRIC("onewaystreams", "Total number of 1-way streams", UINT64F, UINT64F,atomic64_get_na(&rtpe_stats->oneway_stream_sess));
+	METRIC("onewaystreams", "Total number of 1-way streams", "%" PRIu64, "%" PRIu64,atomic64_get_na(&rtpe_stats->oneway_stream_sess));
 	PROM("one_way_sessions_total", "counter");
 	METRICva("avgcallduration", "Average call duration", "%.6f", "%.6f seconds", (double) avg_us / 1000000.0);
 	PROM("call_duration_avg", "gauge");
@@ -477,10 +481,10 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 		METRICva("totalcallsduration", "Total calls duration", "%.6f", "%.6f seconds", calls_dur_iv);
 		HEADER(NULL, "");
 
-		METRIC("minmanagedsessions", "Min managed sessions", UINT64F, UINT64F, min_sess_iv);
-		METRIC("maxmanagedsessions", "Max managed sessions", UINT64F, UINT64F, max_sess_iv);
+		METRIC("minmanagedsessions", "Min managed sessions", "%" PRIu64, "%" PRIu64, min_sess_iv);
+		METRIC("maxmanagedsessions", "Max managed sessions", "%" PRIu64, "%" PRIu64, max_sess_iv);
 
-		for (int i = 0; i < NGC_COUNT; i++) {
+		for (int i = 0; i < OP_COUNT; i++) {
 			double min = (double) atomic64_get_na(&rtpe_sampled_graphite_min_max_sampled.min.ng_command_times[i]) / 1000000.0;
 			double max = (double) atomic64_get_na(&rtpe_sampled_graphite_min_max_sampled.max.ng_command_times[i]) / 1000000.0;
 			double avg = (double) atomic64_get_na(&rtpe_sampled_graphite_avg.avg.ng_command_times[i]) / 1000000.0;
@@ -494,7 +498,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 			METRICsva(avg_label, "%.6f", avg);
 		}
 
-		for (int i = 0; i < NGC_COUNT; i++) {
+		for (int i = 0; i < OP_COUNT; i++) {
 			uint64_t min = atomic64_get_na(&rtpe_rate_graphite_min_max_avg_sampled.min.ng_commands[i]);
 			uint64_t max = atomic64_get_na(&rtpe_rate_graphite_min_max_avg_sampled.max.ng_commands[i]);
 			uint64_t avg = atomic64_get_na(&rtpe_rate_graphite_min_max_avg_sampled.avg.ng_commands[i]);
@@ -524,7 +528,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 			(double) atomic64_get_na(&(source)->sums_squared.stat_name) / (divisor * divisor)); \
 	PROM(prefix #stat_name "2_total", "counter"); \
 	PROMLAB(label); \
-	METRIC(#stat_name "_samples_total", "Total number of " name " samples", UINT64F, UINT64F, \
+	METRIC(#stat_name "_samples_total", "Total number of " name " samples", "%" PRIu64, "%" PRIu64, \
 			atomic64_get_na(&(source)->counts.stat_name)); \
 	PROM(prefix #stat_name "_samples_total", "counter"); \
 	PROMLAB(label); \
@@ -549,19 +553,19 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 	STAT_GET_PRINT(rtt_dsct, "discrete round-trip time", 1.0);
 	STAT_GET_PRINT(packetloss, "packet loss", 1.0);
 	STAT_GET_PRINT(jitter_measured, "jitter (measured)", 1.0);
-	METRIC("packets_lost", "Packets lost", UINT64F, UINT64F,
+	METRIC("packets_lost", "Packets lost", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats->packets_lost));
 	PROM("packets_lost", "counter");
-	METRIC("rtp_duplicates", "Duplicate RTP packets", UINT64F, UINT64F,
+	METRIC("rtp_duplicates", "Duplicate RTP packets", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats->rtp_duplicates));
 	PROM("rtp_duplicates", "counter");
-	METRIC("rtp_skips", "RTP sequence skips", UINT64F, UINT64F,
+	METRIC("rtp_skips", "RTP sequence skips", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats->rtp_skips));
 	PROM("rtp_skips", "counter");
-	METRIC("rtp_seq_resets", "RTP sequence resets", UINT64F, UINT64F,
+	METRIC("rtp_seq_resets", "RTP sequence resets", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats->rtp_seq_resets));
 	PROM("rtp_seq_resets", "counter");
-	METRIC("rtp_reordered", "Out-of-order RTP packets", UINT64F, UINT64F,
+	METRIC("rtp_reordered", "Out-of-order RTP packets", "%" PRIu64, "%" PRIu64,
 			atomic64_get_na(&rtpe_stats->rtp_reordered));
 	PROM("rtp_reordered", "counter");
 	HEADER(NULL, "");
@@ -574,7 +578,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 
 	GString *tmp = g_string_new("");
 	g_string_append_printf(tmp, " %20s ", "Proxy");
-	for (int i = 0; i < NGC_COUNT; i++)
+	for (int i = 0; i < OP_COUNT; i++)
 		g_string_append_printf(tmp, "| %10s ", ng_command_strings_short[i]);
 	HEADERl("%s", tmp->str);
 	g_string_free(tmp, TRUE);
@@ -595,7 +599,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 		tmp = g_string_new("");
 		METRICsva("proxy", "\"%s\"", sockaddr_print_buf(&cur->proxy));
 		g_string_append_printf(tmp, " %20s ", sockaddr_print_buf(&cur->proxy));
-		for (int i = 0; i < NGC_COUNT; i++) {
+		for (int i = 0; i < OP_COUNT; i++) {
 			mutex_lock(&cur->cmd[i].lock);
 
 			g_string_append_printf(tmp, "| %10u ", cur->cmd[i].count);
@@ -612,8 +616,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 
 			mn = g_strdup_printf("%sduration", ng_command_strings_short[i]);
 			lw = g_ascii_strdown(mn, -1);
-			METRICs(lw, "%.6f", (double) cur->cmd[i].time.tv_sec +
-					(double) cur->cmd[i].time.tv_usec / 1000000.);
+			METRICs(lw, "%.6f", (double) cur->cmd[i].time / 1000000.);
 			PROM("request_seconds_total", "counter");
 			PROMLAB("proxy=\"%s\",request=\"%s\"", sockaddr_print_buf(&cur->proxy),
 					ng_command_strings[i]);
@@ -638,7 +641,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 
 	HEADER("]", "");
 
-	for (int i = 0; i < NGC_COUNT; i++) {
+	for (int i = 0; i < OP_COUNT; i++) {
 		char *mn = g_strdup_printf("total%scount", ng_command_strings_short[i]);
 		char *lw = g_ascii_strdown(mn, -1);
 		METRICs(lw, "%u", total.cmd[i].count);
@@ -650,7 +653,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 
 	HEADER("interfaces", NULL);
 	HEADER("[", NULL);
-	for (GList *l = all_local_interfaces.head; l; l = l->next) {
+	for (__auto_type l = all_local_interfaces.head; l; l = l->next) {
 		struct local_intf *lif = l->data;
 		// only show first-order interface entries: socket families must match
 		if (lif->logical->preferred_family != lif->spec->local_address.addr.family)
@@ -666,7 +669,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 
 		METRICs("min", "%u", lif->spec->port_pool.min);
 		METRICs("max", "%u", lif->spec->port_pool.max);
-		unsigned int f = g_hash_table_size(lif->spec->port_pool.free_ports_ht);
+		unsigned int f = lif->spec->port_pool.free_ports_q.length;
 		unsigned int r = lif->spec->port_pool.max - lif->spec->port_pool.min + 1;
 		METRICs("used", "%u", r - f);
 		PROM("ports_used", "gauge");
@@ -685,7 +688,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 		HEADER("}", NULL);
 
 #define F(f) \
-		METRICs(#f, UINT64F, atomic64_get_na(&lif->stats->s.f)); \
+		METRICs(#f, "%" PRIu64, atomic64_get_na(&lif->stats->s.f)); \
 		PROM("interface_" #f, "counter"); \
 		PROMLAB("name=\"%s\",address=\"%s\"", lif->logical->name.s, \
 				sockaddr_print_buf(&lif->spec->local_address.addr));
@@ -693,7 +696,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 #undef F
 
 		// expected to be single thread only, so no locking
-		long long time_diff_us;
+		int64_t time_diff_us;
 		struct interface_stats_block *intv_stats
 			= interface_sampled_rate_stats_get(interface_rate_stats, lif, &time_diff_us);
 
@@ -704,7 +707,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 			struct interface_counter_stats diff;
 			interface_counter_calc_diff(&lif->stats->s, &intv_stats->s, &diff);
 
-#define F(f) METRICs(#f, UINT64F, atomic64_get_na(&diff.f));
+#define F(f) METRICs(#f, "%" PRIu64, atomic64_get_na(&diff.f));
 #include "interface_counter_stats_fields.inc"
 #undef F
 
@@ -717,7 +720,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 				struct interface_counter_stats rate;
 				interface_counter_calc_rate_from_diff(time_diff_us, &diff, &rate);
 
-#define F(f) METRICs(#f, UINT64F, atomic64_get_na(&rate.f));
+#define F(f) METRICs(#f, "%" PRIu64, atomic64_get_na(&rate.f));
 #include "interface_counter_stats_fields.inc"
 #undef F
 
@@ -796,7 +799,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 		HEADER("ingress", NULL);
 		HEADER("{", NULL);
 #define F(f) \
-		METRICs(#f, UINT64F, atomic64_get_na(&lif->stats->in.f)); \
+		METRICs(#f, "%" PRIu64, atomic64_get_na(&lif->stats->in.f)); \
 		PROM("interface_" #f, "gauge"); \
 		PROMLAB("name=\"%s\",address=\"%s\",direction=\"ingress\"", lif->logical->name.s, \
 				sockaddr_print_buf(&lif->spec->local_address.addr));
@@ -807,7 +810,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 		HEADER("egress", NULL);
 		HEADER("{", NULL);
 #define F(f) \
-		METRICs(#f, UINT64F, atomic64_get_na(&lif->stats->out.f)); \
+		METRICs(#f, "%" PRIu64, atomic64_get_na(&lif->stats->out.f)); \
 		PROM("interface_" #f, "gauge"); \
 		PROMLAB("name=\"%s\",address=\"%s\",direction=\"egress\"", lif->logical->name.s, \
 				sockaddr_print_buf(&lif->spec->local_address.addr));
@@ -822,7 +825,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 			struct interface_counter_stats_dir diff_in;
 			interface_counter_calc_diff_dir(&lif->stats->in, &intv_stats->in, &diff_in);
 
-#define F(f) METRICs(#f, UINT64F, atomic64_get_na(&diff_in.f));
+#define F(f) METRICs(#f, "%" PRIu64, atomic64_get_na(&diff_in.f));
 #include "interface_counter_stats_fields_dir.inc"
 #undef F
 
@@ -834,7 +837,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 			struct interface_counter_stats_dir diff_out;
 			interface_counter_calc_diff_dir(&lif->stats->out, &intv_stats->out, &diff_out);
 
-#define F(f) METRICs(#f, UINT64F, atomic64_get_na(&diff_out.f));
+#define F(f) METRICs(#f, "%" PRIu64, atomic64_get_na(&diff_out.f));
 #include "interface_counter_stats_fields_dir.inc"
 #undef F
 
@@ -848,7 +851,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 				interface_counter_calc_rate_from_diff_dir(time_diff_us, &diff_in,
 						&rate);
 
-#define F(f) METRICs(#f, UINT64F, atomic64_get_na(&rate.f));
+#define F(f) METRICs(#f, "%" PRIu64, atomic64_get_na(&rate.f));
 #include "interface_counter_stats_fields_dir.inc"
 #undef F
 
@@ -860,7 +863,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 				interface_counter_calc_rate_from_diff_dir(time_diff_us, &diff_out,
 						&rate);
 
-#define F(f) METRICs(#f, UINT64F, atomic64_get_na(&rate.f));
+#define F(f) METRICs(#f, "%" PRIu64, atomic64_get_na(&rate.f));
 #include "interface_counter_stats_fields_dir.inc"
 #undef F
 
@@ -876,7 +879,7 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 	HEADER("transcoders", NULL);
 	HEADER("[", "");
 
-	int last_tv_sec = rtpe_now.tv_sec - 1;
+	int last_tv_sec = rtpe_now / 1000000 - 1;
 	unsigned int idx = last_tv_sec & 1;
 
 	codec_stats_ht_iter iter;
@@ -890,17 +893,17 @@ stats_metric_q *statistics_gather_metrics(struct interface_sampled_rate_stats *i
 		PROM("transcoders", "gauge");
 		PROMLAB("chain=\"%s\"", chain);
 		if (g_atomic_int_get(&stats_entry->last_tv_sec[idx]) == last_tv_sec) {
-			METRICs("packetrate", UINT64F, atomic64_get(&stats_entry->packets_input[idx]));
-			METRICs("byterate", UINT64F, atomic64_get(&stats_entry->bytes_input[idx]));
-			METRICs("samplerate", UINT64F, atomic64_get(&stats_entry->pcm_samples[idx]));
+			METRICs("packetrate", "%" PRIu64, atomic64_get(&stats_entry->packets_input[idx]));
+			METRICs("byterate", "%" PRIu64, atomic64_get(&stats_entry->bytes_input[idx]));
+			METRICs("samplerate", "%" PRIu64, atomic64_get(&stats_entry->pcm_samples[idx]));
 		}
-		METRICs("packets", UINT64F, atomic64_get(&stats_entry->packets_input[2]));
+		METRICs("packets", "%" PRIu64, atomic64_get(&stats_entry->packets_input[2]));
 		PROM("transcode_packets_total", "counter");
 		PROMLAB("chain=\"%s\"", chain);
-		METRICs("bytes", UINT64F, atomic64_get(&stats_entry->bytes_input[2]));
+		METRICs("bytes", "%" PRIu64, atomic64_get(&stats_entry->bytes_input[2]));
 		PROM("transcode_bytes_total", "counter");
 		PROMLAB("chain=\"%s\"", chain);
-		METRICs("samples", UINT64F, atomic64_get(&stats_entry->pcm_samples[2]));
+		METRICs("samples", "%" PRIu64, atomic64_get(&stats_entry->pcm_samples[2]));
 		PROM("transcode_samples_total", "counter");
 		PROMLAB("chain=\"%s\"", chain);
 		HEADER("}", "");
@@ -922,7 +925,7 @@ static void free_stats_metric(stats_metric *m) {
 	g_free(m->value_short);
 	g_free(m->value_raw);
 	g_free(m->prom_label);
-	g_slice_free1(sizeof(*m), m);
+	g_free(m);
 }
 
 void statistics_free_metrics(stats_metric_q *q) {
@@ -937,28 +940,29 @@ void statistics_free(void) {
 }
 
 static void codec_stats_free(struct codec_stats *stats_entry) {
-	free(stats_entry->chain);
+	g_free(stats_entry->chain);
 	g_free(stats_entry->chain_brief);
-	g_slice_free1(sizeof(*stats_entry), stats_entry);
+	g_free(stats_entry);
 }
 
 TYPED_GHASHTABLE_IMPL(codec_stats_ht, c_str_hash, c_str_equal, NULL, codec_stats_free)
 
 void statistics_init(void) {
-	gettimeofday(&rtpe_started, NULL);
+	rtpe_started = now_us();
 	rtpe_stats = bufferpool_alloc0(shm_bufferpool, sizeof(*rtpe_stats));
 
 	mutex_init(&rtpe_codec_stats_lock);
 	rtpe_codec_stats = codec_stats_ht_new();
 }
 
-const char *statistics_ng(bencode_item_t *input, bencode_item_t *output) {
+const char *statistics_ng(ng_command_ctx_t *ctx) {
 	g_autoptr(stats_metric_q) metrics = statistics_gather_metrics(NULL);
 	g_auto(GQueue) bstack = G_QUEUE_INIT;
 
-	bencode_item_t *dict = output;
+	parser_arg dict = ctx->resp;
+	ng_parser_ctx_t *parser_ctx = &ctx->parser_ctx;
+	const ng_parser_t *parser = parser_ctx->parser;
 	const char *sub_label = "statistics"; // top level
-	bencode_buffer_t *buf = output->buffer;
 
 	for (__auto_type l = metrics->head; l; l = l->next) {
 		stats_metric *m = l->data;
@@ -968,21 +972,21 @@ const char *statistics_ng(bencode_item_t *input, bencode_item_t *output) {
 		// key:value entry?
 		if (m->value_short) {
 			if (m->is_int)
-				bencode_dictionary_add_integer(dict, bencode_strdup(buf, m->label),
+				parser->dict_add_int(dict, parser->strdup(parser_ctx, m->label),
 						m->int_value);
 			else if (m->value_raw)
-				bencode_dictionary_add_string_dup(dict, bencode_strdup(buf, m->label),
-						m->value_raw);
+				parser->dict_add_str_dup(dict, parser->strdup(parser_ctx, m->label),
+						STR_PTR(m->value_raw));
 			else
-				bencode_dictionary_add_string_dup(dict, bencode_strdup(buf, m->label),
-						m->value_short);
+				parser->dict_add_str_dup(dict, parser->strdup(parser_ctx, m->label),
+						STR_PTR(m->value_short));
 			continue;
 		}
 
 		// list or dict end?
 		if (m->is_close_bracket) {
-			dict = g_queue_pop_tail(&bstack);
-			assert(dict != NULL);
+			dict.gen = g_queue_pop_tail(&bstack);
+			assert(dict.gen != NULL);
 			continue;
 		}
 
@@ -994,26 +998,26 @@ const char *statistics_ng(bencode_item_t *input, bencode_item_t *output) {
 		}
 
 		// open bracket of some sort - new sub-entry follows
-		bencode_item_t *sub = NULL;
+		parser_arg sub = {0};
 		if (m->is_brace)
-			sub = bencode_dictionary(buf);
+			sub = parser->dict(parser_ctx);
 		else
-			sub = bencode_list(buf);
+			sub = parser->list(parser_ctx);
 
-		assert(sub != NULL);
+		assert(sub.gen != NULL);
 
 		// is this a dictionary?
-		if (dict->type == BENCODE_DICTIONARY) {
+		if (parser->is_dict(dict)) {
 			assert(sub_label != NULL);
-			bencode_dictionary_add(dict, bencode_strdup(buf, sub_label), sub);
+			parser->dict_add(dict, parser->strdup(parser_ctx, sub_label), sub);
 		}
-		else if (dict->type == BENCODE_LIST)
-			bencode_list_add(dict, sub);
+		else if (parser->is_list(dict))
+			parser->list_add(dict, sub);
 		else
 			abort();
 
 		sub_label = NULL;
-		g_queue_push_tail(&bstack, dict);
+		g_queue_push_tail(&bstack, dict.gen);
 		dict = sub;
 	}
 
@@ -1024,12 +1028,12 @@ const char *statistics_ng(bencode_item_t *input, bencode_item_t *output) {
  * Separate thread for update of running min/max call counters.
  */
 enum thread_looper_action call_rate_stats_updater(void) {
-	static struct timeval last_run;
+	static int64_t last_run;
 
 	stats_rate_min_max(&rtpe_rate_graphite_min_max, &rtpe_stats_rate);
 
-	if (last_run.tv_sec) { /* `stats_counters_calc_rate()` shouldn't be called on the very first cycle */
-		long long run_diff_us = timeval_diff(&rtpe_now, &last_run);
+	if (last_run) { /* `stats_counters_calc_rate()` shouldn't be called on the very first cycle */
+		int64_t run_diff_us = rtpe_now - last_run;
 		stats_counters_calc_rate(rtpe_stats, run_diff_us, &rtpe_stats_intv, &rtpe_stats_rate);
 	}
 
