@@ -599,8 +599,7 @@ int stun(const str *b, stream_fd *sfd, const endpoint_t *sin) {
 	if (class == STUN_CLASS_INDICATION)
 		return 0;
 
-	attr_str.s = &b->s[20];
-	attr_str.len = b->len - 20;
+	attr_str = STR_LEN(&b->s[20], b->len - 20);
 	if (stun_attributes(&attrs, &attr_str, unknowns, req)) {
 		err = "failed to parse attributes";
 		if (unknowns[0] == 0xffff)
@@ -685,7 +684,8 @@ int stun_binding_request(const endpoint_t *dst, uint32_t transaction[3], str *pw
 		str ufrags[2], int controlling, uint64_t tiebreaker, uint32_t priority,
 		socket_t *sock, int to_use)
 {
-	struct async_stun_req *r = uring_alloc_buffer_req(sizeof(*r));
+	struct async_stun_req r_s;
+	struct async_stun_req *r = uring_alloc(&r_s, uring_req_free);
 	int i;
 
 	output_init(&r->mh, r->iov, &r->hdr, STUN_BINDING_REQUEST, transaction);
@@ -710,7 +710,7 @@ int stun_binding_request(const endpoint_t *dst, uint32_t transaction[3], str *pw
 	fingerprint(&r->mh, &r->fp);
 
 	output_finish_src(&r->mh);
-	uring_sendmsg(sock, &r->mh, dst, &r->sin, &r->req);
+	uring_methods.sendmsg(sock, &r->mh, dst, &r->sin, &r->req);
 
 	return 0;
 }
