@@ -19,7 +19,7 @@
 
 struct media_packet;
 struct transport_protocol;
-struct ssrc_ctx;
+struct ssrc_entry_call;
 struct rtpengine_srtp;
 struct jb_packet;
 struct poller;
@@ -29,7 +29,7 @@ TYPED_GQUEUE(stream_fd, stream_fd)
 
 
 typedef int rtcp_filter_func(struct media_packet *, GQueue *);
-typedef int (*rewrite_func)(str *, struct packet_stream *, struct ssrc_ctx *);
+typedef int (*rewrite_func)(str *, struct packet_stream *, struct ssrc_entry_call *);
 
 
 enum transport_protocol_index {
@@ -74,21 +74,39 @@ struct streamhandler {
 
 
 
+TYPED_GQUEUE(local_intf, struct local_intf)
+TYPED_GHASHTABLE(rr_specs_ht, str, struct logical_intf, str_hash, str_equal, NULL, NULL)
+
 struct logical_intf {
 	str				name;
 	sockfamily_t			*preferred_family;
-	GQueue				list; /* struct local_intf */
-	GHashTable			*rr_specs;
+	local_intf_q			list;
+	rr_specs_ht			rr_specs;
 	str				name_base; // if name is "foo:bar", this is "foo"
 };
+
+typedef void port_t;
+TYPED_GQUEUE(ports, port_t)
+
+struct socket_port_link {
+	socket_t			socket;
+	ports_q				links;
+	struct port_pool		*pp;
+};
+
+TYPED_GQUEUE(port_pool, struct port_pool)
 struct port_pool {
 	unsigned int			min, max;
 
 	mutex_t				free_list_lock;
 
-	GQueue				free_ports_q;		/* for getting the next free port */
-	GHashTable			* free_ports_ht;	/* for a lookup, if the port is used */
+	ports_q				free_ports_q;		/* for getting the next free port */
+	ports_list			**free_ports;		/* for a lookup if the port is used */
+
+	port_pool_q			overlaps;
 };
+#define free_ports_link(pp, port) ((pp)->free_ports[port - (pp)->min])
+
 struct intf_address {
 	socktype_t			*type;
 	sockaddr_t			addr;
@@ -101,6 +119,7 @@ struct intf_config {
 	struct intf_address		local_address;
 	struct intf_address		advertised_address;
 	unsigned int			port_min, port_max;
+	GList				*exclude_ports;
 };
 struct intf_spec {
 	struct intf_address		local_address;
@@ -135,13 +154,13 @@ INLINE void interface_counter_calc_diff_dir(const struct interface_counter_stats
 #include "interface_counter_stats_fields_dir.inc"
 #undef F
 }
-INLINE void interface_counter_calc_rate_from_diff(long long run_diff_us,
+INLINE void interface_counter_calc_rate_from_diff(int64_t run_diff_us,
 		struct interface_counter_stats *diff, struct interface_counter_stats *rate) {
 #define F(x) atomic64_calc_rate_from_diff(run_diff_us, atomic64_get(&diff->x), &rate->x);
 #include "interface_counter_stats_fields.inc"
 #undef F
 }
-INLINE void interface_counter_calc_rate_from_diff_dir(long long run_diff_us,
+INLINE void interface_counter_calc_rate_from_diff_dir(int64_t run_diff_us,
 		struct interface_counter_stats_dir *diff, struct interface_counter_stats_dir *rate) {
 #define F(x) atomic64_calc_rate_from_diff(run_diff_us, atomic64_get(&diff->x), &rate->x);
 #include "interface_counter_stats_fields_dir.inc"
@@ -150,26 +169,28 @@ INLINE void interface_counter_calc_rate_from_diff_dir(long long run_diff_us,
 void interface_sampled_rate_stats_init(struct interface_sampled_rate_stats *);
 void interface_sampled_rate_stats_destroy(struct interface_sampled_rate_stats *);
 struct interface_stats_block *interface_sampled_rate_stats_get(struct interface_sampled_rate_stats *s,
-		struct local_intf *lif, long long *time_diff_us);
+		struct local_intf *lif, int64_t *time_diff_us);
+
+TYPED_GQUEUE(socket_port, struct socket_port_link)
 
 struct local_intf {
 	struct intf_spec		*spec;
 	struct intf_address		advertised_address;
 	unsigned int			unique_id; /* starting with 0 - serves as preference */
-	const struct logical_intf	*logical;
+	struct logical_intf		*logical;
 	str				ice_foundation;
 
 	struct interface_stats_block	*stats;
 };
 struct socket_intf_list {
 	struct local_intf		*local_intf;
-	socket_q			list;
+	socket_port_q			list;
 };
 struct sfd_intf_list {
 	struct local_intf		*local_intf;
 	stream_fd_q			list;
 };
-TYPED_GQUEUE(socket_intf_list, struct socket_intf_list)
+TYPED_GQUEUE(socket_intf_list, struct socket_intf_list) /* RO */
 TYPED_GQUEUE(sfd_intf_list, struct sfd_intf_list)
 
 /**
@@ -194,7 +215,10 @@ struct stream_fd {
 	struct obj			obj;
 
 	unsigned int			unique_id;	/* RO */
-	socket_t			socket;		/* RO */
+	union {
+		socket_t			socket;		/* RO - alias */
+		struct socket_port_link		spl;		/* RO */
+	};
 	struct local_intf		*local_intf;	/* RO */
 
 	/* stream_fd object holds a reference to the call it belongs to.
@@ -204,7 +228,7 @@ struct stream_fd {
 	 * The call is only released when it has been dissociated from all stream_fd objects,
 	 * which happens during call teardown.
 	 */
-	call_t			*call;		/* RO */
+	call_t				*call;		/* RO */
 	struct packet_stream		*stream;	/* LOCK: call->master_lock */
 	struct crypto_context		crypto;		/* IN direction, LOCK: stream->in_lock */
 	struct dtls_connection		dtls;		/* LOCK: stream->in_lock */
@@ -214,13 +238,14 @@ struct stream_fd {
 };
 
 struct sink_attrs {
+	// cannot be bit fields because G_STRUCT_OFFSET is used on them
 	bool block_media;
 	bool silence_media;
 
-	unsigned int offer_answer:1; // bidirectional, exclusive
-	unsigned int rtcp_only:1;
-	unsigned int transcoding:1;
-	unsigned int egress:1;
+	bool offer_answer:1; // bidirectional, exclusive
+	bool rtcp_only:1;
+	bool transcoding:1;
+	bool egress:1;
 };
 
 /**
@@ -237,7 +262,7 @@ struct media_packet {
 	str raw;
 
 	endpoint_t fsin; // source address of received packet
-	struct timeval tv; // timestamp when packet was received
+	int64_t tv; // timestamp when packet was received
 	stream_fd *sfd; // fd which received the packet
 	call_t *call; // sfd->call
 	struct packet_stream *stream; // sfd->stream
@@ -248,7 +273,7 @@ struct media_packet {
 
 	struct rtp_header *rtp;
 	struct rtcp_packet *rtcp;
-	struct ssrc_ctx *ssrc_in, *ssrc_out; // SSRC contexts from in_srtp and out_srtp
+	struct ssrc_entry_call *ssrc_in, *ssrc_out; // SSRC contexts from in_srtp and out_srtp
 	str payload;
 
 	codec_packet_q packets_out;
@@ -257,7 +282,7 @@ struct media_packet {
 
 
 
-extern GQueue all_local_interfaces; // read-only during runtime
+extern local_intf_q all_local_interfaces; // read-only during runtime
 
 extern __thread struct bufferpool *media_bufferpool;
 
@@ -268,16 +293,14 @@ void interfaces_free(void);
 struct logical_intf *get_logical_interface(const str *name, sockfamily_t *fam, int num_ports);
 struct local_intf *get_interface_address(const struct logical_intf *lif, sockfamily_t *fam);
 struct local_intf *get_any_interface_address(const struct logical_intf *lif, sockfamily_t *fam);
-void interfaces_exclude_port(unsigned int port);
-int is_local_endpoint(const struct intf_address *addr, unsigned int port);
+void interfaces_exclude_port(endpoint_t *);
+bool is_local_endpoint(const struct intf_address *addr, unsigned int port);
 
-//int get_port(socket_t *r, unsigned int port, const struct local_intf *lif, const call_t *c);
-//void release_port(socket_t *r, const struct local_intf *);
-
-int __get_consecutive_ports(socket_q *out, unsigned int num_ports, unsigned int wanted_start_port,
-		struct intf_spec *spec, const str *);
-int get_consecutive_ports(socket_intf_list_q *out, unsigned int num_ports, unsigned int num_intfs, struct call_media *media);
-stream_fd *stream_fd_new(socket_t *fd, call_t *call, struct local_intf *lif);
+struct socket_port_link get_specific_port(unsigned int port,
+		struct intf_spec *spec, const str *label);
+bool get_consecutive_ports(socket_intf_list_q *out, unsigned int num_ports, unsigned int num_intfs,
+		struct call_media *media);
+stream_fd *stream_fd_new(struct socket_port_link *, call_t *call, struct local_intf *lif);
 stream_fd *stream_fd_lookup(const endpoint_t *);
 void stream_fd_release(stream_fd *);
 enum thread_looper_action release_closed_sockets(void);
@@ -287,18 +310,14 @@ void free_sfd_intf_list(struct sfd_intf_list *il);
 void free_release_sfd_intf_list(struct sfd_intf_list *il);
 void free_socket_intf_list(struct socket_intf_list *il);
 
-INLINE int open_intf_socket(socket_t *r, unsigned int port, const struct local_intf *lif) {
-	return open_socket(r, SOCK_DGRAM, port, &lif->spec->local_address.addr);
-}
-
 void __unkernelize(struct packet_stream *, const char *);
 void unkernelize(struct packet_stream *, const char *);
 void __stream_unconfirm(struct packet_stream *, const char *);
 void __reset_sink_handlers(struct packet_stream *);
 
-int __hunt_ssrc_ctx_idx(uint32_t ssrc, struct ssrc_ctx *list[RTPE_NUM_SSRC_TRACKING],
+int __hunt_ssrc_ctx_idx(uint32_t ssrc, struct ssrc_entry_call *list[RTPE_NUM_SSRC_TRACKING],
 		unsigned int start_idx);
-struct ssrc_ctx *__hunt_ssrc_ctx(uint32_t ssrc, struct ssrc_ctx *list[RTPE_NUM_SSRC_TRACKING],
+struct ssrc_entry_call *__hunt_ssrc_ctx(uint32_t ssrc, struct ssrc_entry_call *list[RTPE_NUM_SSRC_TRACKING],
 		unsigned int start_idx);
 
 void media_packet_copy(struct media_packet *, const struct media_packet *);
