@@ -49,7 +49,7 @@ struct stream_stats {
 	atomic64			packets;
 	atomic64			bytes;
 	atomic64			errors;
-	atomic64			last_packet;
+	atomic64			last_packet_us;
 	atomic_t			tos;
 };
 
@@ -71,7 +71,7 @@ struct ssrc_stats {
 	atomic_t		total_lost;
 	atomic_t		transit;
 	atomic_t		jitter;
-	atomic64		last_packet;
+	atomic64		last_packet_us;
 	atomic_t		last_pt;
 };
 
