@@ -27,7 +27,7 @@ struct streambuf {
 	GString		*buf;
 	void		*fd_ptr;
 	struct poller	*poller;
-	time_t		active;
+	int64_t		active_us;
 	int		eof;
 	const struct streambuf_funcs
 			*funcs;
@@ -41,9 +41,9 @@ void streambuf_destroy(struct streambuf *);
 int streambuf_writeable(struct streambuf *);
 int streambuf_readable(struct streambuf *);
 char *streambuf_getline(struct streambuf *);
-unsigned int streambuf_bufsize(struct streambuf *);
-void streambuf_printf(struct streambuf *, const char *, ...) __attribute__ ((format (printf, 2, 3)));
-void streambuf_vprintf(struct streambuf *, const char *, va_list);
+size_t streambuf_bufsize(struct streambuf *);
+size_t streambuf_printf(struct streambuf *, const char *, ...) __attribute__ ((format (printf, 2, 3)));
+size_t streambuf_vprintf(struct streambuf *, const char *, va_list);
 void streambuf_write(struct streambuf *, const char *, unsigned int);
 INLINE void streambuf_write_str(struct streambuf *b, str *s) {
 	streambuf_write(b, s->s, s->len);
