@@ -157,8 +157,8 @@ static const struct testparams testparams[] = {
 
 
 // settings
-static char *source_codec = "PCMA";
-static char *dest_codec = "opus";
+static char *source_codec;
+static char *dest_codec;
 static int init_threads = 0;
 static gboolean bidirectional = false;
 static int max_cpu = 0;
@@ -168,6 +168,7 @@ static int measure_time = 500;
 static int repeats = 1;
 static gboolean cpu_freq;
 static int freq_granularity = 50;
+static bool codec_chain;
 
 
 #define BLOCKED_COLOR 1
@@ -245,11 +246,6 @@ static pthread_t thread_new(const char *name, void *(*fn)(void *), void *p) {
 static inline long long us_ticks_scale(long long val) {
 	return val * ticks_per_sec / 1000000;
 }
-static inline long long now_us(void) {
-	struct timeval now;
-	gettimeofday(&now, NULL);
-	return timeval_us(&now);
-}
 
 
 // stream is locked
@@ -297,8 +293,6 @@ static int got_packet_pkt(struct stream *s, AVPacket *pkt) {
 		}
 	}
 
-	av_packet_unref(pkt);
-
 	return 0;
 }
 
@@ -371,14 +365,16 @@ static void readable(int fd, void *o) {
 				s->fixture_idx = 0;
 
 			str frame;
-			str_init_len(&frame, (char *) data->data, data->size);
+			frame = STR_LEN(data->data, data->size);
 
 			if (!s->chain)
 				decoder_input_data(s->decoder, &frame, s->input_ts, got_frame, s, NULL);
 			else {
 				AVPacket *pkt = codec_cc_input_data(s->chain, &frame, s->input_ts, s, NULL, NULL);
-				if (pkt)
+				if (pkt) {
 					got_packet_pkt(s, pkt);
+					av_packet_unref(pkt);
+				}
 				else
 					mutex_lock(&s->lock); // was unlocked by async_init
 			}
@@ -404,7 +400,7 @@ static void closed(int fd, void *o) {
 
 static void new_threads(uint num) {
 	while (num--) {
-		struct worker *w = g_slice_alloc0(sizeof(*w));
+		struct worker *w = g_new0(__typeof(*w), 1);
 
 		mutex_init(&w->comput_lock);
 
@@ -433,19 +429,19 @@ static void kill_threads(uint num) {
 
 	while ((w = g_queue_pop_head(&to_join))) {
 		pthread_join(w->thr, NULL);
-		g_slice_free1(sizeof(*w), w);
+		g_free(w);
 	}
 }
 
 
-static void stream_free(void *p) {
-	struct stream *s = p;
+static void stream_free(struct stream *s) {
 	close(s->output_fd);
 	dump_close(s);
 	if (s->encoder)
 		encoder_free(s->encoder);
 	if (s->decoder)
 		decoder_close(s->decoder);
+	codec_cc_free(&s->chain);
 	g_free(s->type);
 }
 
@@ -464,7 +460,6 @@ static void async_finish(AVPacket *pkt, void *async_cb_obj) {
 		got_packet_pkt(s, pkt);
 	}
 	obj_put(s);
-	av_packet_free(&pkt);
 }
 
 static void new_stream_params(
@@ -473,7 +468,7 @@ static void new_stream_params(
 		const codec_def_t *out_def,
 		const struct testparams *outprm
 ) {
-	struct stream *s = obj_alloc0("stream", sizeof(*s), stream_free);
+	__auto_type s = obj_alloc0(struct stream, stream_free);
 
 	// create timerfd
 	s->timer_fd = timerfd_create(CLOCK_REALTIME, TFD_NONBLOCK);
@@ -665,7 +660,7 @@ static void show_popup(const char *fmt, ...) {
 
 	// split into lines and get max line length
 	GQueue lines = G_QUEUE_INIT;
-	str st = STR_INIT(s);
+	str st = STR(s);
 	str token;
 	uint llen = 0;
 	while (str_token_sep(&token, &st, '\n')) {
@@ -1185,7 +1180,7 @@ TYPED_GQUEUE(stats, struct stats)
 static void stats_queue_free(stats_q *q) {
 	struct stats *sp;
 	while ((sp = t_queue_pop_head(q)))
-		g_slice_free1(sizeof(*sp), sp);
+		g_free(sp);
 	t_queue_free(q);
 }
 
@@ -1232,7 +1227,7 @@ static bool cpu_collect(stats_q *outp, struct stats *totals) {
 		}
 
 		if (outp) {
-			struct stats *sp = g_slice_alloc(sizeof(*sp));
+			struct stats *sp = g_new(__typeof(*sp), 1);
 			*sp = stats;
 			t_queue_push_tail(outp, sp);
 		}
@@ -1307,7 +1302,7 @@ static void delay_bar(const struct delay_stats *stats, int line, int x, int brea
 
 
 static void other_thread_free(struct other_thread *thr) {
-	g_slice_free1(sizeof(*thr), thr);
+	g_free(thr);
 }
 
 static int pid_compare(const void *a, const void *b) {
@@ -1362,7 +1357,7 @@ static int other_threads_collect(const bool do_output, int starty, int maxy, int
 		// object already exists?
 		struct other_thread *thr = g_hash_table_lookup(other_threads, GINT_TO_POINTER(pid));
 		if (!thr) {
-			thr = g_slice_alloc0(sizeof(*thr));
+			thr = g_new0(__typeof(*thr), 1);
 			g_hash_table_insert(other_threads, GINT_TO_POINTER(pid), thr);
 		}
 
@@ -1385,7 +1380,7 @@ static int other_threads_collect(const bool do_output, int starty, int maxy, int
 	if (do_output) {
 		// output based on sorted list
 		GQueue threads = G_QUEUE_INIT;
-		g_tree_get_values(&threads, tree);
+		rtpe_g_tree_get_values(&threads, tree);
 
 		int width;
 		bar_grid(threads.length, maxx, &height, &width);
@@ -1693,6 +1688,12 @@ static void options(int *argc, char ***argv) {
 			.description = "Granularity in ms for measuring CPU frequencies",
 			.arg_description = "INT",
 		},
+		{
+			.long_name = "codec-chain",
+			.arg = G_OPTION_ARG_NONE,
+			.arg_data = &codec_chain,
+			.description = "Use codec-chain facilities",
+		},
 		{ NULL, }
 	};
 
@@ -1704,6 +1705,11 @@ static void options(int *argc, char ***argv) {
 	if (init_threads <= 0)
 		init_threads = 1;
 
+	if (!source_codec)
+		source_codec = g_strdup("PCMA");
+	if (!dest_codec)
+		dest_codec = g_strdup("opus");
+
 	if (max_cpu > 100 || max_cpu < 0)
 		die("Invalid `max-cpu` number given");
 	if (freq_granularity <= 0)
@@ -1941,10 +1947,10 @@ int main(int argc, char **argv) {
 	if (!rtpe_poller)
 		die("Failed to create poller");
 
-	decoder_def = codec_find(&STR_INIT(source_codec), MT_AUDIO);
+	decoder_def = codec_find(STR_PTR(source_codec), MT_AUDIO);
 	if (!decoder_def)
 		die("Codec definition for source codec not found");
-	encoder_def = codec_find(&STR_INIT(dest_codec), MT_AUDIO);
+	encoder_def = codec_find(STR_PTR(dest_codec), MT_AUDIO);
 	if (!encoder_def)
 		die("Codec definition for destination codec not found");
 
@@ -1953,6 +1959,20 @@ int main(int argc, char **argv) {
 	if (!find_params(&out_params, dest_codec))
 		die("Definition for output fixture not found");
 
+	if (codec_chain) {
+		cc_init_chain(
+				decoder_def,
+				&(format_t) {
+					.channels = in_params.channels,
+					.clockrate = in_params.clock_rate,
+				},
+				encoder_def,
+				&(format_t) {
+					.channels = out_params.channels,
+					.clockrate = out_params.clock_rate,
+		});
+	}
+
 	load_fixture(&in_params);
 	if (bidirectional)
 		load_fixture(&out_params);
