@@ -5,10 +5,7 @@
 #include "str.h"
 
 
-extern int resample_audio;
-
-
-decode_t *decoder_new(const char *payload_str, const char *format, int ptime, output_t *);
+decode_t *decoder_new(const char *payload_str, const char *format, int ptime);
 int decoder_input(decode_t *, const str *, unsigned long ts, ssrc_t *);
 void decoder_free(decode_t *);
 
