@@ -115,7 +115,7 @@ a=s-foo:bar
 a=s-baz:quux blah
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
-a=label:1
+a=label:0
 a=rtpmap:0 PCMU/8000
 a=m-dummy
 a=m-foo:bar
@@ -154,7 +154,7 @@ a=s-foo:bar
 a=s-baz:quux blah
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
-a=label:1
+a=label:0
 a=rtpmap:0 PCMU/8000
 a=m-dummy
 a=m-foo:bar
@@ -193,7 +193,7 @@ a=s-foo:bar
 a=s-baz:quux blah
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
-a=label:1
+a=label:0
 a=rtpmap:0 PCMU/8000
 a=m-dummy
 a=m-foo:bar
@@ -233,7 +233,7 @@ a=s-foo:bar
 a=s-baz:quux blah
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
-a=label:1
+a=label:0
 a=rtpmap:0 PCMU/8000
 a=m-dummy
 a=m-foo:bar
@@ -272,7 +272,7 @@ a=s-foo:bar
 a=s-baz:quux blah
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
-a=label:1
+a=label:0
 a=rtpmap:0 PCMU/8000
 a=m-dummy
 a=m-foo:bar
@@ -882,10 +882,10 @@ a=rtcp:PORT
 SDP
 
 is $ftr, undef, 'from-tag matches';
-is_deeply $fts, [tt(), ft()], 'from-tags match';
+is_deeply $fts, [ft(), tt()], 'from-tags match';
 is_deeply $tag_medias, [
 	{
-		tag => tt(),
+		tag => ft(),
 		medias => [
 			{
 				index => 1,
@@ -896,7 +896,7 @@ is_deeply $tag_medias, [
 		],
 	},
 	{
-		tag => ft(),
+		tag => tt(),
 		medias => [
 			{
 				index => 1,
@@ -911,13 +911,13 @@ is_deeply $media_labels, {
 	'1' => {
 		index => 1,
 		type => 'audio',
-		tag => ft(),
+		tag => tt(),
 		mode => 'sendrecv',
 	},
 	'0' => {
 		index => 1,
 		type => 'audio',
-		tag => tt(),
+		tag => ft(),
 		mode => 'sendrecv',
 	},
 }, 'media-labels match';
@@ -990,11 +990,11 @@ a=rtcp:PORT
 SDP
 
 is $ftr, undef, 'from-tag matches';
-is_deeply $fts, [tt(), ft()], 'from-tags match';
+is_deeply $fts, [ft(), tt()], 'from-tags match';
 is_deeply $tag_medias, [
 	{
-		tag => tt(),
-		label => 'called',
+		tag => ft(),
+		label => 'caller',
 		medias => [
 			{
 				index => 1,
@@ -1005,8 +1005,8 @@ is_deeply $tag_medias, [
 		],
 	},
 	{
-		tag => ft(),
-		label => 'caller',
+		tag => tt(),
+		label => 'called',
 		medias => [
 			{
 				index => 1,
@@ -1021,15 +1021,15 @@ is_deeply $media_labels, {
 	'1' => {
 		index => 1,
 		type => 'audio',
-		tag => ft(),
-		label => 'caller',
+		tag => tt(),
+		label => 'called',
 		mode => 'sendrecv',
 	},
 	'0' => {
 		index => 1,
 		type => 'audio',
-		tag => tt(),
-		label => 'called',
+		tag => ft(),
+		label => 'caller',
 		mode => 'sendrecv',
 	},
 }, 'media-labels match';
@@ -1090,7 +1090,7 @@ rcv($sock_a, $port_b, rtpm(0, 2000, 4000, 0x3456, "\x00" x 160));
 snd($sock_a, $port_b, rtp(0, 4000, 7000, 0x6543, "\x00" x 160));
 rcv($sock_b, $port_a, rtpm(0, 4000, 7000, 0x6543, "\x00" x 160));
 
-($ftr, $ttr, $fts, undef, undef, $port_d, undef, $port_c) = subscribe_request('"all" sub',
+($ftr, $ttr, $fts, undef, undef, $port_c, undef, $port_d) = subscribe_request('"all" sub',
 	{ 'flags' => ['all'] }, <<SDP);
 v=0
 o=- SDP_VERSION IN IP4 198.51.100.1
@@ -1111,7 +1111,7 @@ a=rtcp:PORT
 SDP
 
 is $ftr, undef, 'from-tag matches';
-is_deeply $fts, [tt(), ft()], 'from-tags match';
+is_deeply $fts, [ft(), tt()], 'from-tags match';
 
 subscribe_answer('"all" sub',
 	{ 'to-tag' => $ttr, flags => ['allow transcoding'] }, <<SDP);
@@ -1119,10 +1119,10 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
-m=audio 6086 RTP/AVP 8
+m=audio 6084 RTP/AVP 8
 c=IN IP4 198.51.100.14
 a=recvonly
-m=audio 6084 RTP/AVP 8
+m=audio 6086 RTP/AVP 8
 c=IN IP4 198.51.100.14
 a=recvonly
 SDP
@@ -1760,7 +1760,7 @@ rcv($sock_a, $port_b, rtpm(0, 2000, 4000, 0x3456, "\x00" x 160));
 snd($sock_a, $port_b, rtp(0, 4000, 7000, 0x6543, "\x00" x 160));
 rcv($sock_b, $port_a, rtpm(0, 4000, 7000, 0x6543, "\x00" x 160));
 
-(undef, $ttr, undef, undef, undef, $port_d, undef, $port_c) = subscribe_request('SIPREC sub pause/resume',
+(undef, $ttr, undef, undef, undef, $port_c, undef, $port_d) = subscribe_request('SIPREC sub pause/resume',
 	{ flags => ['all', 'SIPREC'] }, <<SDP);
 v=0
 o=- SDP_VERSION IN IP4 198.51.100.1
@@ -1786,11 +1786,11 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
-m=audio 6124 RTP/AVP 0
+m=audio 6122 RTP/AVP 0
 c=IN IP4 198.51.100.14
 a=label:0
 a=recvonly
-m=audio 6122 RTP/AVP 0
+m=audio 6124 RTP/AVP 0
 c=IN IP4 198.51.100.14
 a=label:1
 a=recvonly
@@ -1850,7 +1850,7 @@ a=recvonly
 a=rtcp:PORT
 SDP
 
-(undef, undef, undef, $tag_medias, $media_labels, $port_d, undef, $port_c) = subscribe_request('SIPREC sub pause/resume',
+(undef, undef, undef, $tag_medias, $media_labels, $port_c, undef, $port_d) = subscribe_request('SIPREC sub pause/resume',
 	{ flags => ['all', 'SIPREC'], 'to-tag' => $ttr }, <<SDP);
 v=0
 o=- SDP_VERSION IN IP4 198.51.100.1
@@ -1860,48 +1860,48 @@ m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
 a=label:0
 a=rtpmap:0 PCMU/8000
-a=sendonly
+a=inactive
 a=rtcp:PORT
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
 a=label:1
 a=rtpmap:0 PCMU/8000
-a=inactive
+a=sendonly
 a=rtcp:PORT
 SDP
 
 is_deeply $tag_medias, [
+	{
+		tag => ft(),
+		medias => [
+			{
+				index => 1,
+				type => 'audio',
+				label => '0',
+				mode => 'sendonly',
+			},
+		],
+	},
 	{
 		tag => tt(),
-		medias => [
-			{
-				index => 1,
-				type => 'audio',
-				label => '0',
-				mode => 'recvonly',
-			},
-		],
-	},
-	{
-		tag => ft(),
 		medias => [
 			{
 				index => 1,
 				type => 'audio',
 				label => '1',
-				mode => 'sendonly',
+				mode => 'recvonly',
 			},
 		],
 	},
 ], 'SIPREC sub pause/resume - tag-medias match';
 is_deeply $media_labels, {
-	'1' => {
+	'0' => {
 		index => 1,
 		type => 'audio',
 		tag => ft(),
 		mode => 'sendonly',
 	},
-	'0' => {
+	'1' => {
 		index => 1,
 		type => 'audio',
 		tag => tt(),
@@ -1915,14 +1915,14 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
-m=audio 6124 RTP/AVP 0
-c=IN IP4 198.51.100.14
-a=label:0
-a=recvonly
 m=audio 6122 RTP/AVP 0
 c=IN IP4 198.51.100.14
-a=label:1
+a=label:0
 a=inactive
+m=audio 6124 RTP/AVP 0
+c=IN IP4 198.51.100.14
+a=label:1
+a=recvonly
 SDP
 
 snd($sock_b, $port_a, rtp(0, 2001, 4320, 0x3456, "\x00" x 160));
@@ -1971,7 +1971,7 @@ a=sendrecv
 a=rtcp:PORT
 SDP
 
-(undef, $ttr, undef, $tag_medias, $media_labels, $port_d, undef, $port_c) = subscribe_request('SIPREC sub pause/resume',
+(undef, $ttr, undef, $tag_medias, $media_labels, $port_c, undef, $port_d) = subscribe_request('SIPREC sub pause/resume',
 	{ flags => ['all', 'SIPREC'], 'to-tag' => $ttr }, <<SDP);
 v=0
 o=- SDP_VERSION IN IP4 198.51.100.1
@@ -1993,7 +1993,7 @@ SDP
 
 is_deeply $tag_medias, [
 	{
-		tag => tt(),
+		tag => ft(),
 		medias => [
 			{
 				index => 1,
@@ -2004,7 +2004,7 @@ is_deeply $tag_medias, [
 		],
 	},
 	{
-		tag => ft(),
+		tag => tt(),
 		medias => [
 			{
 				index => 1,
@@ -2016,13 +2016,13 @@ is_deeply $tag_medias, [
 	},
 ], 'SIPREC sub pause/resume - tag-medias match';
 is_deeply $media_labels, {
-	'1' => {
+	'0' => {
 		index => 1,
 		type => 'audio',
 		tag => ft(),
 		mode => 'sendrecv',
 	},
-	'0' => {
+	'1' => {
 		index => 1,
 		type => 'audio',
 		tag => tt(),
@@ -2036,11 +2036,11 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
-m=audio 6124 RTP/AVP 0
+m=audio 6122 RTP/AVP 0
 c=IN IP4 198.51.100.14
 a=label:0
 a=recvonly
-m=audio 6122 RTP/AVP 0
+m=audio 6124 RTP/AVP 0
 c=IN IP4 198.51.100.14
 a=label:1
 a=recvonly
@@ -2106,48 +2106,48 @@ m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
 a=label:0
 a=rtpmap:0 PCMU/8000
-a=inactive
+a=sendonly
 a=rtcp:PORT
 m=audio PORT RTP/AVP 0
 c=IN IP4 203.0.113.1
 a=label:1
 a=rtpmap:0 PCMU/8000
-a=sendonly
+a=inactive
 a=rtcp:PORT
 SDP
 
 is_deeply $tag_medias, [
+	{
+		tag => ft(),
+		medias => [
+			{
+				index => 1,
+				type => 'audio',
+				label => '0',
+				mode => 'recvonly',
+			},
+		],
+	},
 	{
 		tag => tt(),
-		medias => [
-			{
-				index => 1,
-				type => 'audio',
-				label => '0',
-				mode => 'sendonly',
-			},
-		],
-	},
-	{
-		tag => ft(),
 		medias => [
 			{
 				index => 1,
 				type => 'audio',
 				label => '1',
-				mode => 'recvonly',
+				mode => 'sendonly',
 			},
 		],
 	},
 ], 'tag-medias match';
 is_deeply $media_labels, {
-	'1' => {
+	'0' => {
 		index => 1,
 		type => 'audio',
 		tag => ft(),
 		mode => 'recvonly',
 	},
-	'0' => {
+	'1' => {
 		index => 1,
 		type => 'audio',
 		tag => tt(),
@@ -2161,14 +2161,14 @@ v=0
 o=- 1545997027 1 IN IP4 198.51.100.1
 s=tester
 t=0 0
-m=audio 6124 RTP/AVP 0
-c=IN IP4 198.51.100.14
-a=label:0
-a=inactive
 m=audio 6122 RTP/AVP 0
 c=IN IP4 198.51.100.14
-a=label:1
+a=label:0
 a=recvonly
+m=audio 6124 RTP/AVP 0
+c=IN IP4 198.51.100.14
+a=label:1
+a=inactive
 SDP
 
 snd($sock_a, $port_b, rtp(0, 4001, 7480, 0x6543, "\x00" x 160));
